
/* Chrysalide - Outil d'analyse de fichiers binaires
 * module.c - intégration du répertoire gtkext en tant que module
 *
 * Copyright (C) 2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "module.h"


#include <assert.h>


#include "blockview.h"
#include "bufferview.h"
#include "viewpanel.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Ajoute le module 'gtkext' au module Python.                  *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool add_gtkext_module_to_python_module(PyObject *super)
{
    bool result;                            /* Bilan à retourner           */
    PyObject *module;                       /* Sous-module mis en place    */
    int ret;                                /* Bilan d'un appel            */

    static PyModuleDef py_chrysalide_gtkext_module = {

        .m_base = PyModuleDef_HEAD_INIT,

        .m_name = "pychrysalide.gtkext",
        .m_doc = "Python module for Chrysalide.gtkext",

        .m_size = -1,

    };

    result = false;

    module = PyModule_Create(&py_chrysalide_gtkext_module);
    if (module == NULL) return false;

    ret = PyState_AddModule(super, &py_chrysalide_gtkext_module);
    if (ret != 0) goto agmtpm_exit;

    ret = _PyImport_FixupBuiltin(module, "pychrysalide.gtkext");
    if (ret != 0) goto agmtpm_exit;

    Py_INCREF(module);
    ret = PyModule_AddObject(super, "gtkext", module);
    if (ret != 0) goto agmtpm_exit;

    result = true;

    result &= register_python_view_panel(module);
    result &= register_python_buffer_view(module);
    result &= register_python_block_view(module);

 agmtpm_exit:

    assert(result);

    return result;

}
