
/* Chrysalide - Outil d'analyse de fichiers binaires
 * if.c - décompilation des branchements conditionnels
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include "../instruction.h"
#include "../../../decomp/expr/comp.h"
#include "../../../decomp/expr/cond.h"
#include "../../../decomp/expr/immediate.h"
#include "../../../decomp/instr/ite.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de branchement conditionnel.       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_if(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    CompSignType sign;                      /* Type d'opération menée      */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *op1;                   /* Premier opérande utilisé    */
    GDecInstruction *op2;                   /* Second opérande utilisé     */
    vmpa_t jmp;                             /* Adresse de saut             */
    GDecInstruction *comp;                  /* Comparaison à restituer     */
    GDecInstruction *cond;                  /* Transformation en condition */

    switch (g_dalvik_instruction_get_opcode(G_DALVIK_INSTRUCTION(instr)))
    {
        case DOP_IF_EQ:
            sign = CST_EQ;
            break;
        case DOP_IF_NE:
            sign = CST_NE;
            break;
        case DOP_IF_LT:
            sign = CST_LT;
            break;
        case DOP_IF_GE:
            sign = CST_GE;
            break;
        case DOP_IF_GT:
            sign = CST_GT;
            break;
        case DOP_IF_LE:
            sign = CST_LE;
            break;
        default:
            sign = CST_COUNT;
            break;
    }

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    op1 = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    op2 = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_arch_instruction_get_operand(instr, 2);
    jmp = 0x1234ull;/*g_dec_context_convert_register(ctx, operand, addr);*/

    comp = g_comp_expression_new(G_DEC_EXPRESSION(op1), sign, G_DEC_EXPRESSION(op2));
    cond = g_cond_expression_new(comp);
    result = g_ite_instruction_new(cond, jmp, jmp);

    return result;

}



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de branchement conditionnel.       *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_if_zero(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    CompSignType sign;                      /* Type d'opération menée      */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *op1;                   /* Premier opérande utilisé    */
    GDecInstruction *op2;                   /* Second opérande utilisé     */
    vmpa_t jmp;                             /* Adresse de saut             */
    GDecInstruction *comp;                  /* Comparaison à restituer     */
    GDecInstruction *cond;                  /* Transformation en condition */

    switch (g_dalvik_instruction_get_opcode(G_DALVIK_INSTRUCTION(instr)))
    {
        case DOP_IF_EQZ:
            sign = CST_EQ;
            break;
        case DOP_IF_NEZ:
            sign = CST_NE;
            break;
        case DOP_IF_LTZ:
            sign = CST_LT;
            break;
        case DOP_IF_GEZ:
            sign = CST_GE;
            break;
        case DOP_IF_GTZ:
            sign = CST_GT;
            break;
        case DOP_IF_LEZ:
            sign = CST_LE;
            break;
        default:
            sign = CST_COUNT;
            break;
    }

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 0);
    op1 = g_dec_context_convert_register(ctx, operand, false, addr);

    operand = g_imm_operand_new_from_value(MDS_32_BITS_UNSIGNED, (unsigned int)0);
    op2 = g_imm_expression_new(G_IMM_OPERAND(operand));

    operand = g_arch_instruction_get_operand(instr, 2);
    jmp = 0x1234ull;/*g_dec_context_convert_register(ctx, operand, addr);*/

    comp = g_comp_expression_new(G_DEC_EXPRESSION(op1), sign, G_DEC_EXPRESSION(op2));
    cond = g_cond_expression_new(comp);
    result = g_ite_instruction_new(cond, jmp, jmp);

    return result;

}
