
/* Chrysalide - Outil d'analyse de fichiers binaires
 * arithm.h - prototypes pour la représentation des opérations arithmétiques
 *
 * Copyright (C) 2010 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_DECOMP_COMMON_ARITHM_H
#define _ANALYSIS_DECOMP_COMMON_ARITHM_H


#include <glib-object.h>


#include "../expression.h"
#include "../instruction.h"



#define G_TYPE_ARITHM_EXPRESSION               g_arithm_expression_get_type()
#define G_ARITHM_EXPRESSION(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_arithm_expression_get_type(), GArithmExpression))
#define G_IS_ARITHM_EXPRESSION(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_arithm_expression_get_type()))
#define G_ARITHM_EXPRESSION_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_ARITHM_EXPRESSION, GArithmExpressionClass))
#define G_IS_ARITHM_EXPRESSION_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_ARITHM_EXPRESSION))
#define G_ARITHM_EXPRESSION_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_ARITHM_EXPRESSION, GArithmExpressionClass))


/* Définition d'une opération arithmétique définie (instance) */
typedef struct _GArithmExpression GArithmExpression;

/* Définition d'une opération arithmétique définie (classe) */
typedef struct _GArithmExpressionClass GArithmExpressionClass;


/* Types d'opérations menables */
typedef enum _ArithmOperationType
{
    AOT_ADD,                                /* Addition                    */
    AOT_SUB,                                /* Soustraction                */
    AOT_MUL,                                /* Multiplication              */
    AOT_DIV,                                /* Division                    */
    AOT_REM,                                /* Modulo                      */
    AOT_AND,                                /* Et logique                  */
    AOT_OR,                                 /* Ou logique                  */
    AOT_XOR,                                /* Ou exclusif                 */

    AOT_COUNT

} ArithmOperationType;


/* Indique le type défini pour une opération arithmétique définie. */
GType g_arithm_expression_get_type(void);

/* Représente une opération arithmétique entre deux opérandes. */
GDecInstruction *g_arithm_expression_new(GDecExpression *, ArithmOperationType, GDecExpression *);



#endif  /* _ANALYSIS_DECOMP_COMMON_ARITHM_H */
